#################################################################################################
#sequential.py
#Author: Swetha Varadarajan
#Last Update by Swetha Varadarajan: Oct 20, 2018
#Usage: python sequential.py
#################################################################################################
#More details:
#Polyhedral Optimizations of RNA-RNA Interaction Computations
#MS Thesis: Swetha Vardarajan, Defended: Oct 26,2017
#Advisor: Sanjay Rajopadhye
#################################################################################################
#For more details, contact: varadarajan.swetha@gmail.com
#################################################################################################

import numpy as np
import matplotlib.pyplot as plt
import math
from textwrap import wrap
 
# data to plot
data = np.loadtxt('sequential.txt')
n_groups = data.shape[0] 
 
# create plot
fig, ax = plt.subplots()
index = np.arange(n_groups)
bar_width = 0.5
opacity = 0.8

#plt.plot(data[:,2], 'm--', label='Ospsq_Baseline_Haswell_Sequential', linewidth=4)
#plt.plot(data[:,4], 'r--', label='Ospsq_Baseline_Broadwell_Sequential', linewidth=4)

rects1 = plt.bar(index*4, data[:,1]/data[:,4], bar_width,color='b',
                 label='OSPSQ-best-row')
 
rects2 = plt.bar(index*4 + bar_width, data[:,2]/data[:,4], bar_width,color='y',
                 label='OSPSQ-best-col')
 
rects3 = plt.bar(index*4 + bar_width*2, data[:,3]/data[:,4], bar_width,color='r',
                 label='OSPSQ-vect')

rects4 = plt.bar(index*4 + bar_width*3, data[:,4]/data[:,4], bar_width,color='g',
                 label='OSPSQ-base')

plt.xlabel('N')
plt.ylabel('GFLOPS/s Normalized to OSPSQ-base version')
#plt.title('Ospsq_sq Baseline: Parallel performance on Haswell & Broadwell architectures\n for a problem size of N=30 and M=1000')
#title = ax.set_title("\n".join(wrap("Ospsq_sq Baseline: Parallel performance on Haswell and Broadwell architectures for a problem size of N=30 and M=2000", 60)))

plt.xticks(index*4 + bar_width*2, data[:,0].astype(int))
#plt.legend()
plt.legend(bbox_to_anchor=(0., 1.02, 1., .102), loc=3,
           ncol=2, mode="expand", borderaxespad=0.)
 
plt.ylim(0,25)
plt.axvline(x=18.8,color='k', linestyle='--')
plt.axvline(x=38.8,color='k', linestyle='--')
plt.axvline(x=58.8,color='k', linestyle='--')
plt.axvline(x=78.8,color='k', linestyle='--')

plt.text(5,23.5,'M=750')
plt.text(25,23.5,'M=1000')
plt.text(45,23.5,'M=1500')
plt.text(65,23.5,'M=2000')
plt.text(85,23.5,'M=2500')

fig.tight_layout()
#title.set_y(1.05)
fig.subplots_adjust(top=0.8)
plt.savefig('seq.png')

