#################################################################################################
#thesis.cs
#Date Created: September 17,2017
#Author: Swetha Varadarajan
#Last Update by Swetha Varadarajan: Oct 20, 2018
#################################################################################################
#More details:
#Polyhedral Optimizations of RNA-RNA Interaction Computations
#MS Thesis: Swetha Vardarajan, Defended: Oct 26,2017
#Advisor: Sanjay Rajopadhye
#################################################################################################
#For more details, contact: varadarajan.swetha@gmail.com
#################################################################################################

prog1 = ReadAlphabets("./ospsq_baseline.ab");
system1 = "osp_sq";
CheckProgram(prog1);

#Memory space shared so as to be similar to that of C writing style
setMemoryMap(prog1, system1, "FTable", "Q", "(i1,j1,i2,j2->i1,j1,i2,j2)");
setMemoryMap(prog1, system1, "_serNR_FTable", "Q", "(i1,j1,i2,j2,d,e->i1,j1,i2,j2)");

#Flattened arrays
options=createCGOptionForScheduledC();
setCGOptionFlattenArrays(options,1);

################################################################################################
#OSPSQ-base#####################################################################################

setSpaceTimeMap(prog1, system1, "FTable", "(i1,j1,i2,j2->j1-i1,j2-i2,i1,i2,j1,j2)");
setSpaceTimeMap(prog1, system1, "_serNR_FTable", "(i1,j1,i2,j2,d,e->j1-i1,j2-i2,i1,i2,d,e)");

outDir1 = "../test-out/ospsq-base";
generateScheduledCode(prog1, system1, options, outDir1);
generateWrapper(prog1, system1, options, outDir1);
generateMakefile(prog1, system1, options, outDir1);

setParallel(prog1,system1,"",2);
outDir2 = "../test-out/ospsq-base-par";
generateScheduledCode(prog1, system1, options, outDir2);
generateWrapper(prog1, system1, options, outDir2);
generateMakefile(prog1, system1, options, outDir2);

#################################################################################################
#ospsq-vect######################################################################################

setSpaceTimeMap(prog1, system1, "FTable", "(i1,j1,i2,j2->j1-i1,i1,i2,j1,j2,j2-i2)");
setSpaceTimeMap(prog1, system1, "_serNR_FTable", "(i1,j1,i2,j2,d,e->j1-i1,i1,i2,d,e,j2-i2)");

outDir1 = "../test-out/ospsq-vect";
generateScheduledCode(prog1, system1, options, outDir1);
generateWrapper(prog1, system1, options, outDir1);
generateMakefile(prog1, system1, options, outDir1);

#################################################################################################
#ospsq-best-col######################################################################################

setSpaceTimeMap(prog1, system1, "FTable", "(i1,j1,i2,j2->j1-i1,i1,j1,i2,j2,j2-i2)");
setSpaceTimeMap(prog1, system1, "_serNR_FTable", "(i1,j1,i2,j2,d,e->j1-i1,i1,d,i2,e,j2-i2)");

outDir1 = "../test-out/ospsq-best-col";
generateScheduledCode(prog1, system1, options, outDir1);
generateWrapper(prog1, system1, options, outDir1);
generateMakefile(prog1, system1, options, outDir1);

outDir2 = "../test-out/ospsq-best-col-fine";
setParallel(prog1,system1,"",3);
generateScheduledCode(prog1, system1, options, outDir2);
generateWrapper(prog1, system1, options, outDir2);
generateMakefile(prog1, system1, options, outDir2);

outDir2 = "../test-out/ospsq-best-col-coarse";
setParallel(prog1,system1,"",1);
generateScheduledCode(prog1, system1, options, outDir2);
generateWrapper(prog1, system1, options, outDir2);
generateMakefile(prog1, system1, options, outDir2);

#################################################################################################
#ospsq-best-row######################################################################################

setSpaceTimeMap(prog1, system1, "FTable", "(i1,j1,i2,j2->-i1,j1,j1,-i2,j2,j2)");
setSpaceTimeMap(prog1, system1, "_serNR_FTable", "(i1,j1,i2,j2,d,e->-i1,j1,d,-i2,e,j2)");

outDir1 = "../test-out/ospsq-best-row";
generateScheduledCode(prog1, system1, options, outDir1);
generateWrapper(prog1, system1, options, outDir1);
generateMakefile(prog1, system1, options, outDir1);

outDir2 = "../test-out/ospsq-best-row-par";
setParallel(prog1,system1,"",3);
generateScheduledCode(prog1, system1, options, outDir2);
generateWrapper(prog1, system1, options, outDir2);
generateMakefile(prog1, system1, options, outDir2);

#################################################################################################
